/*
 * byojundo.c
 * byojeopardy
 *
 * by Doug Wolfe
 * doug@wolfescience.com 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "byojundo.h"
#include "callbacks.h"
#include "interface.h"
#include <xhtmlite/xhtmlite_module/support.h>

static GList *undo_list = NULL;
static GList *redo_list = NULL;

static GtkTreeView *byoj_undo_treeview = NULL;
static GtkWidget *byoj_menuitem_undo = NULL;
static GtkWidget *byoj_menuitem_redo = NULL;

static void 
byoj_undo_store_value(GtkTreeModel *tree_model, GtkTreeIter * iter, undo * undo_data)
{
	value_struct * value_data = malloc(sizeof(value_struct));
	gchar * value_name;
	gtk_tree_model_get (tree_model, iter, 0, &value_name, -1);	
	value_data->value_string = g_strdup(value_name);
	value_data->position = byoj_get_value_position(tree_model, iter);
	if(gtk_tree_model_iter_has_child(tree_model, iter))
	{
		int c;
		for (c = 0; c < gtk_tree_model_iter_n_children(tree_model, iter); c++)
		{
			GtkTreeIter citer;
			gtk_tree_model_iter_nth_child(tree_model, &citer, iter, c);
			int iter_type;
			gtk_tree_model_get (tree_model, &citer, BYOJ_COLUMN_TYPE, &iter_type, -1);
			gchar * question_string;
			gtk_tree_model_get (tree_model, &citer, BYOJ_COLUMN_QUESTION, &question_string, -1);
			if(iter_type == BYOJ_QUESTION)
			{
				printf("added question:%s\n", question_string);
				value_data->question_string = g_strdup(question_string);
			}
			else //answer
			{
				printf("added answer:%s\n", question_string);
				value_data->answer_string = g_strdup(question_string);
			}
		}	
	}
	undo_data->topic_data->values = g_list_append (undo_data->topic_data->values, value_data);
}

static void
byoj_undo_store_data_for_edit(GtkTreeModel *tree_model, GtkTreeIter * iter, undo * undo_data)
{
		int iter_type;
		gtk_tree_model_get (tree_model, iter, BYOJ_COLUMN_TYPE, &iter_type, -1);
		printf("byoj_undo_store_data_for_edit: iter type:%d\n", iter_type);
		undo_data->iter_type = iter_type;
		gchar * column_string; 
		if(undo_data->iter_type == BYOJ_QUESTION || undo_data->iter_type == BYOJ_ANSWER)
		{
			gtk_tree_model_get (tree_model, iter,
				BYOJ_COLUMN_QUESTION, &column_string, -1);
		}
		else
		{
			gtk_tree_model_get (tree_model, iter,
				BYOJ_COLUMN_TOPIC, &column_string, -1);
		}
		undo_data->column_string = g_strdup(column_string);
}

static void
byoj_undo_row_changed(GtkTreeModel *tree_model,
                                            GtkTreePath  *path,
                                            GtkTreeIter  *iter,
                                            gpointer      user_data) 
{
	//printf("***row changed\n");
}

static void
byoj_undo_row_deleted(GtkTreeModel *tree_model,
                                            GtkTreePath  *path,
                                            GtkTreeIter  *iter,
                                            gpointer      user_data)  
{
	//printf("***row deleted\n");

}

static void
byoj_undo_row_has_child_toggled(GtkTreeModel *tree_model,
                                            GtkTreePath  *path,
                                            GtkTreeIter  *iter,
                                            gpointer      user_data)  
{

}

static void
byoj_undo_row_inserted(GtkTreeModel *tree_model,
                                            GtkTreePath  *path,
                                            GtkTreeIter  *iter,
                                            gpointer      user_data)  
{
	//printf("***row inserted\n");
}


static void
byoj_undo_row_reordered(GtkTreeModel *tree_model,
                                            GtkTreePath  *path,
                                            GtkTreeIter  *iter,
                                            gpointer      arg3,
                                            gpointer      user_data)  
{
	//printf("***row reordered\n");
}

static void
byoj_undo_set_buttons()
{
	gtk_widget_set_sensitive (byoj_menuitem_undo, byoj_undo_has_undo());
	gtk_widget_set_sensitive (byoj_menuitem_redo, byoj_undo_has_redo());
}


int
byoj_get_topic_position(GtkTreeModel *tree_model, GtkTreeIter * iter)
{
	gchar * path_string = gtk_tree_model_get_string_from_iter (tree_model, iter);
	printf("path_string:%s\n",path_string);
	printf("topic_position:%d\n",atoi(path_string));
	return atoi(path_string); 
}

int
byoj_get_value_position(GtkTreeModel *tree_model, GtkTreeIter * iter)
{
	gchar * path_string = gtk_tree_model_get_string_from_iter (tree_model, iter);
	printf("path_string:%s\n",path_string);
	gint topic_position;
	gint value_position;
	sscanf(path_string, "%d:%d[^\n]", &topic_position, &value_position);
	printf("topic_position:%d\n",topic_position);
	printf("value_position:%d\n",value_position);
	return value_position; 
}

void
byoj_undo_store_data_for_delete(GtkTreeModel *tree_model, GtkTreeIter * iter, undo * undo_data)
{
	undo_data->topic_data = malloc(sizeof(topic));
	int iter_type;
	gtk_tree_model_get (tree_model, iter, BYOJ_COLUMN_TYPE, &iter_type, -1);
	undo_data->iter_type = iter_type;
	undo_data->topic_data->values = NULL;
	undo_data->topic_data->position = byoj_get_topic_position(tree_model, iter);
	if(iter_type == BYOJ_TOPIC || iter_type == BYOJ_DAILY_DOUBLE)
	{
		gchar *topic_name;
		gtk_tree_model_get (tree_model, iter, BYOJ_COLUMN_TOPIC, &topic_name, -1);
		printf("store topic:%s\n", topic_name);
		undo_data->topic_data->topic_string = g_strdup(topic_name);
		if(gtk_tree_model_iter_has_child(tree_model, iter))
		{
			int c;
			for (c=0;c<gtk_tree_model_iter_n_children(tree_model, iter); c++)
			{
				printf("store value:%d\n", c);
				GtkTreeIter citer;
				gtk_tree_model_iter_nth_child(tree_model, &citer, iter, c);
				byoj_undo_store_value(tree_model, &citer, undo_data);
			}
		return;
		}
	}
	else if(iter_type == BYOJ_VALUE)
	{
		printf("store value\n");
		undo_data->topic_data->topic_string = NULL;
		byoj_undo_store_value(tree_model, iter, undo_data);
	}
	else if(iter_type == BYOJ_QUESTION || iter_type == BYOJ_ANSWER)
	{
		printf("store question or answer\n");
		gchar *question_name;
		gtk_tree_model_get (tree_model, iter, BYOJ_COLUMN_QUESTION, &question_name, -1);
		undo_data->column_string = g_strdup(question_name);
	}
}

void 
byoj_undo_setup(GtkWidget *treeview)
{
	byoj_undo_treeview = GTK_TREE_VIEW(treeview);
	GtkTreeModel * treemodel = gtk_tree_view_get_model (byoj_undo_treeview);

	g_signal_connect ((gpointer) treemodel, "row-changed",
                    G_CALLBACK (byoj_undo_row_changed),
                    NULL);
	g_signal_connect ((gpointer) treemodel, "row-deleted",
                    G_CALLBACK (byoj_undo_row_deleted),
                    NULL);
	g_signal_connect ((gpointer) treemodel, "row-has-child-toggled",
                    G_CALLBACK (byoj_undo_row_has_child_toggled),
                    NULL);
	g_signal_connect ((gpointer) treemodel, "row-inserted",
                    G_CALLBACK (byoj_undo_row_inserted),
                    NULL);
	g_signal_connect ((gpointer) treemodel, "rows-reordered",
                    G_CALLBACK (byoj_undo_row_reordered),
                    NULL);

	byoj_menuitem_undo = lookup_widget(treeview, "byoj_menuitem_undo");
	byoj_menuitem_redo = lookup_widget(treeview, "byoj_menuitem_redo");
	byoj_undo_set_buttons();
}

void
byoj_undo_clear_undos ()
{
//for new files or opened files
	while(g_list_length(undo_list)>0)
	{
		undo * undo_data = g_list_nth_data (undo_list, 0);
		undo_list = g_list_remove(undo_list, undo_data);
	}
	while(g_list_length(redo_list)>0)
	{
		undo * redo_data = g_list_nth_data (redo_list, 0);
		redo_list = g_list_remove(redo_list, redo_data);
	}
	byoj_undo_set_buttons();
}

void
byoj_undo_add_undo (gint undo_type, GtkTreeIter * iter)
{
	printf("add undo\n");
	GtkTreeModel * tree_model = gtk_tree_view_get_model (byoj_undo_treeview);

	undo * undo_data = malloc(sizeof(undo));
	undo_data->action = undo_type;	
	GtkTreePath * path = gtk_tree_model_get_path (tree_model, iter);
	undo_data->path = gtk_tree_path_copy(path);

	//Save Selection
	GtkTreeIter iter_selected;
	GtkTreeSelection * selection = gtk_tree_view_get_selection(byoj_undo_treeview);
	GtkTreePath * path_selected;
	if(gtk_tree_selection_get_selected(selection, &tree_model, &iter_selected))
	{
		path_selected = gtk_tree_model_get_path (tree_model, &iter_selected);
	}
	else
	{
		path_selected = gtk_tree_path_new_first ();
	}
	undo_data->path_selected = gtk_tree_path_copy(path_selected);	

	if(undo_type == UNDO_DELETE)
	{
		byoj_undo_store_data_for_delete(tree_model, iter, undo_data);
	}
	else if(undo_type == UNDO_INSERT)
	{
	}
	else if(undo_type == UNDO_EDIT)
	{
		byoj_undo_store_data_for_edit(tree_model, iter, undo_data);
	}

	undo_list=g_list_prepend (undo_list, undo_data);
	byoj_undo_set_buttons();
}

void
byoj_undo_add_redo (gint undo_type, GtkTreeIter * iter)
{
 	printf("add redo\n");
	GtkTreeModel * tree_model = gtk_tree_view_get_model (byoj_undo_treeview);

	undo * undo_data = malloc(sizeof(undo));
	undo_data->action = undo_type;	
	GtkTreePath * path = gtk_tree_model_get_path (tree_model, iter);
	undo_data->path = gtk_tree_path_copy(path);	

	//Save Selection
	GtkTreeIter iter_selected;
	GtkTreeSelection * selection = gtk_tree_view_get_selection(byoj_undo_treeview);
	GtkTreePath * path_selected;
	if(gtk_tree_selection_get_selected(selection, &tree_model, &iter_selected))
	{
		path_selected = gtk_tree_model_get_path (tree_model, &iter_selected);
	}
	else
	{
		path_selected = gtk_tree_path_new_first ();
	}
	undo_data->path_selected = gtk_tree_path_copy(path_selected);	


	if(undo_type == UNDO_DELETE)
	{
		byoj_undo_store_data_for_delete(tree_model, iter, undo_data);
	}
	else if(undo_type == UNDO_INSERT)
	{
	}
	else if(undo_type == UNDO_EDIT)
	{
		byoj_undo_store_data_for_edit(tree_model, iter, undo_data);
	}

	redo_list=g_list_prepend (redo_list, undo_data);
}

int
byoj_undo_has_undo ()
{
	printf("has undo:");
	if(g_list_length(undo_list)>0)
	{
		printf("TRUE\n");
		return 1;
	}
	printf("FALSE\n");
	return 0;                    
}

int
byoj_undo_has_redo ()
{
	printf("has redo:");
	if(g_list_length(redo_list)>0)
	{
		printf("TRUE\n");
		return 1;
	}
	printf("FALSE\n");
	return 0; 
}

void
byoj_undo_undo ()
{
 if(!byoj_undo_has_undo())
 {return;}              
	undo * undo_data = g_list_nth_data (undo_list, 0);
	GtkTreeModel * tree_model = gtk_tree_view_get_model (byoj_undo_treeview);
	GtkTreeIter iter_selected;
	GtkTreeSelection * selection = gtk_tree_view_get_selection(byoj_undo_treeview);

	if(undo_data->action == UNDO_DELETE)
	{
		printf("path restoring=%s\n", gtk_tree_path_to_string(undo_data->path));
		GtkTreeIter topic_iter;
		gboolean has_topic_iter = FALSE;
		if(undo_data->iter_type == BYOJ_TOPIC)
		{
			has_topic_iter = TRUE;
			topic_iter = byoj_add_topic(GTK_TREE_STORE(tree_model), undo_data->topic_data->topic_string, undo_data->topic_data->position);
		}
		else if(undo_data->iter_type == BYOJ_DAILY_DOUBLE)
		{
			//check to see if there is a daily double first
			if(byoj_has_daily_double(byoj_undo_treeview))
			{
				undo_list = g_list_remove(undo_list, undo_data);
				byoj_undo_set_buttons();
				return;
			}
			has_topic_iter = TRUE;
			topic_iter = byoj_add_daily_double_topic(GTK_TREE_STORE(tree_model));
		}
		else
		{
			printf("iter path:%d\n",undo_data->topic_data->position); 
			gtk_tree_model_get_iter_from_string(tree_model, &topic_iter, g_strdup_printf("%d", undo_data->topic_data->position));
		}
		
		int n = 0;
		value_struct * value_data = g_list_nth_data (undo_data->topic_data->values, n);
		GtkTreeIter value_iter;
		while (value_data != NULL)
		{	
			value_iter = byoj_add_value(GTK_TREE_STORE(tree_model), topic_iter, value_data->value_string, value_data->question_string, value_data->answer_string, value_data->position);
			n+=1;
			value_data = g_list_nth_data (undo_data->topic_data->values, n);
		}
		if(has_topic_iter)
		{
			byoj_undo_add_redo(UNDO_INSERT, &topic_iter);
		}
		else
		{
			byoj_undo_add_redo(UNDO_INSERT, &value_iter);
		}
		gtk_tree_view_expand_row (byoj_undo_treeview, undo_data->path_selected, FALSE);
	}
	else if (undo_data->action == UNDO_INSERT)
	{
		GtkTreeIter iter;
        if(gtk_tree_model_get_iter (tree_model, &iter, undo_data->path))
        {
			byoj_undo_add_redo(UNDO_DELETE, &iter);
   			gtk_tree_store_remove(GTK_TREE_STORE(tree_model), &iter);
        }
	}
	else if(undo_data->action == UNDO_EDIT)
	{
		printf("path editing=%s\n", gtk_tree_path_to_string(undo_data->path));
		GtkTreeIter iter;
        if(gtk_tree_model_get_iter (tree_model, &iter, undo_data->path))
        {
			byoj_undo_add_redo(UNDO_EDIT, &iter);
			gchar * column_string;
			if(undo_data->iter_type == BYOJ_QUESTION || undo_data->iter_type == BYOJ_ANSWER)
			{
				gtk_tree_store_set (GTK_TREE_STORE(tree_model), &iter,
					BYOJ_COLUMN_QUESTION, undo_data->column_string, -1);
			}
			else
			{
				gtk_tree_store_set (GTK_TREE_STORE(tree_model), &iter,
					BYOJ_COLUMN_TOPIC, undo_data->column_string, -1);
			}
			undo_data->column_string = g_strdup(column_string);
        }
	}


	//GET Selection
	gtk_tree_selection_select_path  (selection, undo_data->path_selected);

	undo_list = g_list_remove(undo_list, undo_data);
	byoj_undo_set_buttons();
}

void
byoj_undo_redo ()
{
 if(!byoj_undo_has_redo())
 {return;}   
	undo * undo_data = g_list_nth_data (redo_list, 0);
	GtkTreeModel * tree_model = gtk_tree_view_get_model (byoj_undo_treeview);
	GtkTreeIter iter_selected;
	GtkTreeSelection * selection = gtk_tree_view_get_selection(byoj_undo_treeview);

	if(undo_data->action == UNDO_DELETE)
	{
		printf("path restoring=%s\n", gtk_tree_path_to_string(undo_data->path));
		GtkTreeIter topic_iter;
		gboolean has_topic_iter = FALSE;
		if(undo_data->iter_type == BYOJ_TOPIC)
		{
			has_topic_iter = TRUE;
			topic_iter = byoj_add_topic(GTK_TREE_STORE(tree_model), undo_data->topic_data->topic_string, undo_data->topic_data->position);
		}
		else if(undo_data->iter_type == BYOJ_DAILY_DOUBLE)
		{
			//check to see if there is a daily double first
			if(byoj_has_daily_double(byoj_undo_treeview))
			{
				redo_list = g_list_remove(redo_list, undo_data);
				byoj_undo_set_buttons();
				return;
			}
			has_topic_iter = TRUE;
			topic_iter = byoj_add_daily_double_topic(GTK_TREE_STORE(tree_model));
		}
		else
		{
			printf("iter path:%d\n",undo_data->topic_data->position); 
			gtk_tree_model_get_iter_from_string(tree_model, &topic_iter, g_strdup_printf("%d", undo_data->topic_data->position));
		}
		
		int n = 0;
		value_struct *value_data = g_list_nth_data (undo_data->topic_data->values, n);
		GtkTreeIter value_iter;
		while (value_data != NULL)
		{
			value_iter = byoj_add_value(GTK_TREE_STORE(tree_model), topic_iter, value_data->value_string, value_data->question_string, value_data->answer_string, value_data->position);
			n+=1;
			value_data = g_list_nth_data (undo_data->topic_data->values, n);
		}
		if(has_topic_iter)
		{
			byoj_undo_add_undo(UNDO_INSERT, &topic_iter);
		}
		else
		{
			byoj_undo_add_undo(UNDO_INSERT, &value_iter);
		}
		gtk_tree_view_expand_row (byoj_undo_treeview, undo_data->path_selected, FALSE);
	}
	else if(undo_data->action == UNDO_INSERT)
	{
		GtkTreeIter iter;
        if(gtk_tree_model_get_iter (tree_model, &iter, undo_data->path))
        {
   			byoj_undo_add_undo(UNDO_DELETE, &iter);
			gtk_tree_store_remove(GTK_TREE_STORE(tree_model), &iter);
        }
	}
	else if(undo_data->action == UNDO_EDIT)
	{
		printf("path editing=%s\n", gtk_tree_path_to_string(undo_data->path));
		GtkTreeIter iter;
        if(gtk_tree_model_get_iter (tree_model, &iter, undo_data->path))
        {
			byoj_undo_add_undo(UNDO_EDIT, &iter);
			gchar *column_string;
			if(undo_data->iter_type == BYOJ_QUESTION || undo_data->iter_type == BYOJ_ANSWER)
			{
				gtk_tree_store_set (GTK_TREE_STORE(tree_model), &iter,
					BYOJ_COLUMN_QUESTION, undo_data->column_string, -1);
			}
			else
			{
				gtk_tree_store_set (GTK_TREE_STORE(tree_model), &iter,
					BYOJ_COLUMN_TOPIC, undo_data->column_string, -1);
			}
			undo_data->column_string = g_strdup(column_string);
        }
	}

	//GET Selection
	gtk_tree_selection_select_path  (selection, undo_data->path_selected);
	redo_list = g_list_remove(redo_list, undo_data);
	byoj_undo_set_buttons();
}
