/*
 * callbacks.c
 * byojeopardy
 *
 * by Doug Wolfe
 * doug@wolfescience.com 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "callbacks.h"
#include "interface.h"
#include "export.h"
#include "byojundo.h"
#include "byojmusic.h"
#include <xhtmlite/xhtmlite_module/support.h>

//for random number generator
#include <stdlib.h>
#include <time.h>

//for old input/output functions
#include <stdio.h>
#include <string.h>

//for xml
#include <libxml/xmlreader.h>
#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>

/*for xhtmlite dialog*/
#include <xhtmlite/xhtmlite_module/xhtmlcallbacks.h>
#include <xhtmlite/xhtmlite_module/xhtmlinterface.h>
#include <xhtmlite/xhtmlite_module/xhtmlbuffer.h>

#define MY_ENCODING "UTF-8"
//TODO: set to standard gnome practices
GtkWidget *byoj_window_main;
gboolean byoj_answer_in_form_of_question = TRUE;

static gboolean unsaved;

static int
byoj_read_ignore_whitespace(xmlTextReaderPtr reader)
{
 int ret = 1;
 ret = xmlTextReaderRead(reader);
 if(xmlTextReaderNodeType(reader) == XML_READER_TYPE_SIGNIFICANT_WHITESPACE )
 {
     byoj_read_ignore_whitespace(reader);
 }
 return ret;                  
}

static void
byoj_set_title(gboolean Edited)
{
	gchar *window_title = g_strdup_printf("BYOJeopardy");
	if(current_filename != NULL && strlen(current_filename)!=0)  
	{
		if(Edited)
		{
			window_title = g_strdup_printf("*%s -- BYOJeopardy", g_filename_display_basename(current_filename));
		}
		else
		{
			window_title = g_strdup_printf("%s -- BYOJeopardy", g_filename_display_basename(current_filename));
		}
	}
	else
	{
		window_title = g_strdup_printf("*Untitled -- BYOJeopardy");
	}
	gtk_window_set_title(GTK_WINDOW(byoj_window_main), window_title);
	unsaved = Edited;  //Todo: add prompt to save file before closing for unsaved files.
}

static gchar *
byoj_question_size(gchar *question_string)
{
   int len = strlen(question_string);
   const gchar relative_size[15];
   //printf("question length:%d\n", len);
   if(len > 58  && len < 100)
   {
       g_sprintf(relative_size, "medium");
   }
   else if (len > 100)
   {
       g_sprintf(relative_size, "small");
   }
   else
   {
       g_sprintf(relative_size, "large");
   }
   
   return g_strdup(relative_size);
}

static void 
byoj_adjust_default_tag_table(GtkTextBuffer *textbuffer)
{
       //adjust tags in default tag table for the larger display
	GtkTextTagTable *textbuffer_tagtable = gtk_text_buffer_get_tag_table (textbuffer);
	GtkTextTag *tag;
    tag = gtk_text_tag_table_lookup(textbuffer_tagtable, "sup");
	g_object_set (tag,
		      "rise", PANGO_SCALE * 20, //20 for htmllite fonts
		      "size", PANGO_SCALE * 8,
		      NULL);

	tag = gtk_text_tag_table_lookup(textbuffer_tagtable, "sub");
	g_object_set (tag,
		      "rise", PANGO_SCALE * -20, //-6 for htmllite fonts
		      "size", PANGO_SCALE * 8,
		      NULL);

	tag = gtk_text_tag_table_lookup(textbuffer_tagtable, "span font-size='xx-small'");
	g_object_set (tag,
		      "size", PANGO_SCALE * 20,   //PANGO_SCALE * 7 for htmllite
		      NULL);

	tag = gtk_text_tag_table_lookup(textbuffer_tagtable, "span font-size='x-small'");
	g_object_set (tag,
		      "size", PANGO_SCALE * 28,   //PANGO_SCALE * 8 for htmllite
		      NULL);

	tag = gtk_text_tag_table_lookup(textbuffer_tagtable, "span font-size='small'");
	g_object_set (tag,
		      "size", PANGO_SCALE * 36,   //PANGO_SCALE * 10 for htmllite
		      NULL);
		      
	tag = gtk_text_tag_table_lookup(textbuffer_tagtable, "span font-size='medium'");
	g_object_set (tag,
		      "size", PANGO_SCALE * 48,   //PANGO_SCALE * 12 for htmllite
		      NULL);
 
	tag = gtk_text_tag_table_lookup(textbuffer_tagtable, "span font-size='large'");
	g_object_set (tag,
		      "size", PANGO_SCALE * 72,    //PANGO_SCALE * 16 for htmllite
		      NULL);

	tag = gtk_text_tag_table_lookup(textbuffer_tagtable, "span font-size='x-large'");
	g_object_set (tag,
		      "size", PANGO_SCALE * 96,    //PANGO_SCALE * 24 for htmllite
		      NULL);

	tag = gtk_text_tag_table_lookup(textbuffer_tagtable, "span font-size='xx-large'");
	g_object_set (tag,
		      "size", PANGO_SCALE * 128,    //PANGO_SCALE * 36 for htmllite
		      NULL);  
}

static void
byoj_show_board(GtkWidget *widget)
{
	//this should just make the board visible
	GtkWidget *byoj_table = lookup_widget(widget, "byoj_table");
	GtkWidget *byoj_image = lookup_widget(widget, "byoj_image");
	GtkWidget *byoj_hbox_treeview = lookup_widget(widget, "byoj_hbox_treeview");		
	GtkWidget *byoj_menu_edit = lookup_widget(widget, "byoj_menu_edit");
	gtk_widget_show (byoj_table);
	gtk_widget_hide_all (byoj_menu_edit);
	gtk_widget_hide (byoj_image);
	gtk_widget_hide (byoj_hbox_treeview);
}

static gboolean
byoj_process_node(xmlTextReaderPtr reader, GtkTreeModel *store) 
{
	const xmlChar *name, *value_string;
//TODO:Test reading with errors in code
	name = xmlTextReaderConstName(reader);
	if (name == NULL)
		name = BAD_CAST "--";
	
	value_string = xmlTextReaderConstValue(reader);

	if(!xmlStrcasecmp(name, (const xmlChar*)"topic") && xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT)
	{
		GtkTreeIter piter;
		gtk_tree_store_append (GTK_TREE_STORE(store), &piter, NULL);
		
		int ret = 1;
		while (ret==1)
		{
              ret = byoj_read_ignore_whitespace(reader);
              name = xmlTextReaderConstName(reader);
		if(!xmlStrcasecmp(name, (const xmlChar*)"topic_name") &&   xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT)
		{
             ret = byoj_read_ignore_whitespace(reader);
			value_string = xmlTextReaderConstValue(reader);
			if(value_string == NULL  && ret < 1)
			{
				printf("Error opening file: Unable to read topic\n");
				return FALSE;
			}
			if(!xmlStrcasecmp(value_string, (const xmlChar*)"Daily Double"))
			{
							printf("*************************got daily double****************\n");
				gtk_tree_store_set (GTK_TREE_STORE(store), &piter,
						BYOJ_COLUMN_TOPIC, value_string,
						BYOJ_COLUMN_TOPIC_EDITABLE, FALSE,
						BYOJ_COLUMN_TYPE, BYOJ_DAILY_DOUBLE,
						-1);
			}
			else
			{
				gtk_tree_store_set (GTK_TREE_STORE(store), &piter,
						BYOJ_COLUMN_TOPIC, value_string,
						BYOJ_COLUMN_TOPIC_EDITABLE, TRUE,
						BYOJ_COLUMN_TYPE, BYOJ_TOPIC,
						-1);
			}
			printf("Added Topic: %s\n", value_string);
		}
		
        ret = byoj_read_ignore_whitespace(reader);
        ret = byoj_read_ignore_whitespace(reader);
		name = xmlTextReaderConstName(reader);
        while(ret==1)
        {

		if(!xmlStrcasecmp(name, (const xmlChar*)"value") && xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT)
		{
			GtkTreeIter citer, question_iter, answer_iter;
			gtk_tree_store_append (GTK_TREE_STORE(store), &citer, &piter);  /* Acquire an iterator */
			gtk_tree_store_append (GTK_TREE_STORE(store), &question_iter, &citer);  /* Acquire an iterator */
			gtk_tree_store_append (GTK_TREE_STORE(store), &answer_iter, &citer);  /* Acquire an iterator */
			
			int value_node = 1;
			//gchar *value_str, *question_str, *answer_str;
			value_node = byoj_read_ignore_whitespace(reader);
			while (value_node==1)
			{
                  name = xmlTextReaderConstName(reader);
                  value_node = byoj_read_ignore_whitespace(reader);
			      value_string = xmlTextReaderConstValue(reader);
			      printf("value:%s\n", value_string);
			      if(value_string == NULL  && value_node < 1)
			      {
                         printf("Error opening file: Unable to read value\n");
						 return FALSE;
                  }
			
				printf("trying to add:%s\n", name);
				if(!xmlStrcasecmp(name, (const xmlChar*)"amount") && xmlTextReaderNodeType(reader) == XML_READER_TYPE_TEXT )
				{
                    //value_node =  g_strdup((gchar *)node_str);
					gtk_tree_store_set (GTK_TREE_STORE(store), &citer,
							BYOJ_COLUMN_TOPIC, value_string,
							BYOJ_COLUMN_TOPIC_EDITABLE, TRUE,
							BYOJ_COLUMN_TYPE, BYOJ_VALUE,
							-1);
					printf("Added amount: %s\n", value_string);
				}
				if(!xmlStrcasecmp(name, (const xmlChar*)"question") &&   xmlTextReaderNodeType(reader) == XML_READER_TYPE_TEXT )
				{
					//question_str = g_strdup(node_str);
					gtk_tree_store_set (GTK_TREE_STORE(store), &question_iter,
							BYOJ_COLUMN_QUESTION, value_string,
							BYOJ_COLUMN_QUESTION_EDITABLE, TRUE,
							BYOJ_COLUMN_TYPE, BYOJ_QUESTION,
							-1);
					printf("Added question: %s\n", value_string);
				}
				if (!xmlStrcasecmp(name, (const xmlChar*)"answer") &&   xmlTextReaderNodeType(reader) == XML_READER_TYPE_TEXT )
				{
					//answer_str = g_strdup(node_str);
					gtk_tree_store_set (GTK_TREE_STORE(store), &answer_iter,
							BYOJ_COLUMN_QUESTION, value_string,
							BYOJ_COLUMN_QUESTION_EDITABLE, TRUE,
							BYOJ_COLUMN_TYPE, BYOJ_ANSWER,
							-1);
					printf("Added answer: %s\n", value_string);
				}
			
				value_node = byoj_read_ignore_whitespace(reader);
				value_node = byoj_read_ignore_whitespace(reader);
				name = xmlTextReaderConstName(reader);
				printf("trying to process:%s\n", name);
				if (name == NULL)
				{
					name = BAD_CAST "--";
				}
				value_string = xmlTextReaderConstValue(reader);
				if(!xmlStrcasecmp(name, (const xmlChar*)"value") && xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT)
				{
					value_node=0;
				}
			}
			//byoj_add_question(GTK_TREE_STORE(store), piter, value_str, question_str, answer_str);

			printf("leaving value while\n");
		}			
		    ret  = byoj_read_ignore_whitespace(reader);
			name = xmlTextReaderConstName(reader);
			if (name == NULL)
			{
				name = BAD_CAST "--";
			}
			if(!xmlStrcasecmp(name, (const xmlChar*)"topic") && xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT)
			{
				ret=0;
			}
        }
        }
		printf("leaving while\n");
		
	}
	
	printf("%d %d %s %d %d", 
		xmlTextReaderDepth(reader),
		xmlTextReaderNodeType(reader),
		name,
		xmlTextReaderIsEmptyElement(reader),
		xmlTextReaderHasValue(reader));
	if (value_string == NULL)
		printf("\n");
	else {
		if (xmlStrlen(value_string) > 40)
			printf(" %.40s...\n", value_string);
		else
			printf(" %s\n", value_string);
	}
	return TRUE;
}

static gboolean
byoj_read_xml_file(const char *filename, GtkWidget *widget) 
{
	//TODO: Test for BYOJeopardy file version
	GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(widget), "byoj_treeview");
	
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	gtk_tree_store_clear(GTK_TREE_STORE(store));
	
	xmlTextReaderPtr reader;
	int ret;

	reader = xmlReaderForFile(filename, NULL, 0);
	if (reader != NULL)
	{
		ret = byoj_read_ignore_whitespace(reader);
		while (ret == 1) {
			if(!byoj_process_node(reader, store))
			{
				return FALSE;
			}
			ret = xmlTextReaderRead(reader);
		}
		xmlTextReaderRead(reader);
		if (ret != 0) {
			fprintf(stderr, "%s : failed to parse\n", filename);
			return FALSE;
		}
	}
	else
	{
		fprintf(stderr, "Unable to open %s\n", filename);
		return FALSE;	
	}
	return TRUE;
}


static gboolean
byoj_open_text_file (gchar *filename, GtkWidget *widget)
{
	//printf("Attempting to open file: %s\n", filename);
	
	FILE *board; /* declare a FILE pointer  */
	board = fopen(filename, "r");

	/* open byojeopary file for reading */

	if(board==NULL) {
		printf("Error: can't open file.\n");
		return FALSE;
		/* fclose(board); DON'T PASS A NULL POINTER TO fclose !! */
	}
	else {
		//printf("File opened successfully. Contents:\n\n");
		
		GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(widget), "byoj_treeview");
		
		GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
		gtk_tree_store_clear(GTK_TREE_STORE(store));
		
		char line[4096];  /* declare a char array */
		GtkTreeIter   piter;
		while(fgets(line, 4096, board)!=NULL) { 
			/* keep looping until NULL pointer... */
			//printf("Reading line: %s", line);
			/* print the file one line at a time  */

			if(!strstr(line, ";"))
			{
				//topic
				char topic_str[4096]="";
				sscanf( line, "%[^\n]", &topic_str);
				gtk_tree_store_append (GTK_TREE_STORE(store), &piter, NULL);  /* Acquire an iterator */
				if(strcasecmp(topic_str, "Daily Double")==0)
				{
					gtk_tree_store_set (GTK_TREE_STORE(store), &piter,
						BYOJ_COLUMN_QUESTION, topic_str,
						BYOJ_COLUMN_QUESTION_EDITABLE, TRUE,
						BYOJ_COLUMN_TYPE, BYOJ_TOPIC,
						-1);
				}
				else
				{
					gtk_tree_store_set (GTK_TREE_STORE(store), &piter,
						BYOJ_COLUMN_QUESTION, topic_str,
						BYOJ_COLUMN_QUESTION_EDITABLE, FALSE,
						BYOJ_COLUMN_TYPE, BYOJ_DAILY_DOUBLE,
						-1);	
				}
				//printf("Added Topic: %s\n", line);
			}
			else
			{
				char value_str[4096]="";
				char question_str[4096]="";
				char answer_str[4096]="";
				sscanf( line, "%[^;];%[^;];%[^\n]", &value_str, &question_str, &answer_str);
				
				if(strcmp(value_str,"")==0)
				{
					sscanf( line, ";%[^;];%[^\n]", &question_str, &answer_str );			
				}
				if(strcmp(question_str,"")==0 && strcmp(answer_str,"")==0)
				{
					sscanf( line, "%[^;];;%[^\n]", &value_str, &answer_str );
					
				}
				if(strcmp(value_str,"")==0 && strcmp(question_str,"")==0 && strcmp(answer_str,"")==0)
				{
					sscanf( line, ";;%[^\n]", &answer_str );
					
				}
			
				byoj_add_value(GTK_TREE_STORE(store), piter, value_str, question_str, answer_str, -1);
			}
		}
		//printf("\n\nNow closing file...\n");
		fclose(board);
	}
	return TRUE;
}

static void
byoj_button_restore_question_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkWidget *button_table = user_data;
	gtk_widget_set_sensitive(button_table, TRUE);
}

static void
byoj_button_close_question_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkWidget *button_table = user_data;
	gtk_widget_set_sensitive(button_table, FALSE);
}

static gboolean
byoj_eventbox_button_release_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	GtkWidget *button_table = gtk_bin_get_child (GTK_BIN(widget));
	if (event->button == 3)
	{
		//printf("call menu\n");
		GtkWidget *menu = create_byoj_menu_question();
		
		GtkWidget *byoj_button_restore_question = lookup_widget(GTK_WIDGET(menu), "byoj_button_restore_question");
		GtkWidget *byoj_button_close_question = lookup_widget(GTK_WIDGET(menu), "byoj_button_close_question");
		if(button_table->state == GTK_STATE_NORMAL)
		{
			gtk_widget_hide (byoj_button_restore_question);
			gtk_widget_show (byoj_button_close_question);
			g_signal_connect ((gpointer) byoj_button_close_question, "activate",
					   G_CALLBACK (byoj_button_close_question_on_activate),
					button_table);
		}
		else
		{
			gtk_widget_show (byoj_button_restore_question);
			gtk_widget_hide (byoj_button_close_question);
			g_signal_connect ((gpointer) byoj_button_restore_question, "activate",
					G_CALLBACK (byoj_button_restore_question_on_activate),
					button_table);
		}

		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
			event->button, event->time);
	}
	else
	{
		if(button_table->state == GTK_STATE_NORMAL)
		{
			gtk_button_clicked (GTK_BUTTON(button_table));
		}
	}
  return FALSE;
}

static GtkWidget *byoj_window_question = NULL;
static void
byoj_board_button_clicked (GtkButton *button)
{
	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
	GtkWidget *byoj_textview_question;
	GtkTextBuffer *textbuffer;
	
	if(byoj_window_question == NULL)
	{
		byoj_window_question = create_byoj_window_question ();
	
		//create textbuffer
		byoj_textview_question = lookup_widget(GTK_WIDGET(byoj_window_question), "byoj_textview_question");
		textbuffer = xhtmlite_create_buffer(GTK_TEXT_VIEW(byoj_textview_question), FALSE);
		byoj_adjust_default_tag_table(textbuffer);
		gtk_window_fullscreen(GTK_WINDOW(byoj_window_question));
	}
	else
	{
		byoj_textview_question = lookup_widget(GTK_WIDGET(byoj_window_question), "byoj_textview_question");
		textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(byoj_textview_question));
	}

	GtkWidget *byoj_label_topic = lookup_widget(GTK_WIDGET(byoj_window_question), "byoj_label_topic");
	GtkWidget *byoj_label_value = lookup_widget(GTK_WIDGET(byoj_window_question), "byoj_label_value");
	//GtkWidget *btnQuestion = lookup_widget(GTK_WIDGET(byoj_window_question), "btnQuestion");
	GtkWidget *byoj_button_label_question = lookup_widget(GTK_WIDGET(byoj_window_question), "byoj_button_label_question");

	gchar *markup;
	
	gchar *topic = g_object_get_data(G_OBJECT(button), "topic");
	gtk_label_set_text(GTK_LABEL(byoj_label_topic), topic);
	markup = g_strdup_printf("<span size='20000' color='#999'>%s</span>", topic);
	gtk_label_set_markup(GTK_LABEL(byoj_label_topic),markup);
		
	gchar *value = g_object_get_data(G_OBJECT(button), "value");
	gtk_label_set_text(GTK_LABEL(byoj_label_value), value);
	markup = g_strdup_printf("<span size='20000' color='#999'>%s</span>", value);
	gtk_label_set_markup(GTK_LABEL(byoj_label_value),markup);
	
	gchar *question = g_object_get_data(G_OBJECT(button), "question");
	g_object_set_data(G_OBJECT(byoj_window_question), "question", question);
		
	gchar *answer = g_object_get_data(G_OBJECT(button), "answer");
	g_object_set_data(G_OBJECT(byoj_window_question), "answer", answer);

	gboolean *daily_double = g_object_get_data(G_OBJECT(button), "daily_double");
	//g_object_set_data(G_OBJECT(byoj_window_question), "daily_double", daily_double);
	      
    gchar *buf;
	if(daily_double)
	{
		markup = g_strdup_printf("<span size='20000' color='#CC3333'>Start Daily Double</span>", value);
		buf = g_strdup_printf("<center><byojeopardy_large>Daily Double</byojeopardy_large></center>");
		byoj_music_dailydouble_song();
	}
	else
	{
		if(byoj_answer_in_form_of_question)
		{
			markup = g_strdup_printf("<span size='20000' color='#333'>Question</span>");
            gchar * relative_size = byoj_question_size(answer);
            buf = g_strdup_printf("<center><byojeopardy_%s>%s</byojeopardy_%s></center>", relative_size, answer, relative_size);
		}
		else
		{
			markup = g_strdup_printf("<span size='20000' color='#333'>Answer</span>");
            gchar * relative_size = byoj_question_size(question);
            buf = g_strdup_printf("<center><byojeopardy_%s>%s</byojeopardy_%s></center>", relative_size, question, relative_size);
		}
	}
	gtk_label_set_markup(GTK_LABEL(byoj_button_label_question), markup);
	xhtmlite_import(textbuffer, buf, TRUE);
	
	gtk_widget_show (byoj_window_question);
}

/*static int byoj_random_number(int range_min, int range_max)
{
   // Generate random numbers in the half-closed interval
   // [range_min, range_max). In other words,
   // range_min <= random number < range_max
   srand( (unsigned)time( NULL ) );
   int u = (double)rand() / (RAND_MAX + 1) * (range_max - range_min) + range_min;
	u = (double)rand() / (RAND_MAX + 1) * (range_max - range_min) + range_min;
   return u;
}*/

static void
byoj_build_board (GtkWidget *widget)
{
	//open dialog and build board
	//Get Format
	GtkWidget *dialog;
	dialog = create_byoj_dialog_format ();
	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(byoj_window_main));

	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
	{
		gtk_widget_hide(dialog);
		GtkWidget *byoj_radiobutton_form_question = lookup_widget(GTK_WIDGET(dialog), "byoj_radiobutton_form_question");
		byoj_answer_in_form_of_question = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(byoj_radiobutton_form_question));
	}
	gtk_widget_destroy(dialog);
	//build board
	//really a build board method below
    //printf("Start Building Board\n");

	GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(widget), "byoj_treeview");
	GtkWidget *table = lookup_widget(GTK_WIDGET(widget), "byoj_table");
	GtkWidget *byoj_image = lookup_widget(GTK_WIDGET(widget), "byoj_image");
	GtkWidget *byoj_vbox = lookup_widget(GTK_WIDGET(widget), "byoj_vbox");
	GtkWidget *byoj_hbox_treeview = lookup_widget(GTK_WIDGET(widget), "byoj_hbox_treeview");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));

	//find the rows and columns
	int rows = 1;
	int columns = 0;
	int total_blocks_on_board = 0; //for daily double
	gboolean daily_double = FALSE;
	gchar *daily_double_topic;
	gchar *daily_double_question;
	gchar *daily_double_answer;
	// 
	GtkTreeIter iter;
	gboolean more_topics = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
	while (more_topics) {
 		int iter_type;
		gtk_tree_model_get (store, &iter, BYOJ_COLUMN_TYPE, &iter_type, -1);
		if(iter_type != BYOJ_DAILY_DOUBLE)
		{
			columns += 1;
			int children_count = gtk_tree_model_iter_n_children(store, &iter);
			total_blocks_on_board += children_count; //for daily double
			if (children_count > rows)
			{
				rows = children_count;
			}	
		}
		else //daily double
		{
			GtkTreeIter value_iter;
			if(gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &value_iter, &iter, 0))
			{
				GtkTreeIter question_iter;
				gtk_tree_model_get (GTK_TREE_MODEL(store), &value_iter, BYOJ_COLUMN_TOPIC, &daily_double_topic, -1);
				if(gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &question_iter, &value_iter, 0))
				{
					gtk_tree_model_get (GTK_TREE_MODEL(store), &question_iter, BYOJ_COLUMN_QUESTION, &daily_double_question, -1);
				}
				if(gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &question_iter, &value_iter, 1))
				{
					gtk_tree_model_get (GTK_TREE_MODEL(store), &question_iter, BYOJ_COLUMN_QUESTION, &daily_double_answer, -1);
				}
				daily_double = TRUE;
			}
		}
		more_topics =  gtk_tree_model_iter_next (GTK_TREE_MODEL(store), &iter);
	}

	//rebuild table
	gtk_container_foreach (GTK_CONTAINER (table),
			(GtkCallback) gtk_widget_destroy,
			NULL);

	if (columns > 0 && rows > 1)
    {	
	  gtk_table_resize(GTK_TABLE(table), rows, columns);
    }
    else
    {
     //printf("Error opening file: no topics processed\n");
      gtk_widget_hide(table);
      return;  
    }
     
	//get random daily double block
	int daily_double_block = -1; //for no daily double
	if(daily_double)
	{
		GRand *rand = g_rand_new ();
		daily_double_block = g_rand_int_range (rand, 0, total_blocks_on_board - 1);
		//daily_double_block = byoj_random_number(0, total_blocks_on_board-1); //find the percent of random number * the total possible
		//printf("Daily Double Block: %d\n", daily_double_block);
	}

	//add buttons to table
	int column = 0;
	int current_block = 0; //for daily double
	more_topics = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
	while (more_topics) {
		int iter_type;
		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, BYOJ_COLUMN_TYPE, &iter_type, -1);
		if(iter_type == BYOJ_TOPIC)
		{
			gchar *topic_string;
			gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, BYOJ_COLUMN_TOPIC, &topic_string, -1);
			GtkWidget *toplabel = gtk_label_new(topic_string);
			printf("making column label: %s\n", topic_string);
			gtk_label_set_markup(GTK_LABEL(toplabel), (gchar *)g_strdup_printf("<b><span size='14000'>%s</span></b>", topic_string));
			gtk_widget_show(toplabel);
			gtk_label_set_line_wrap(GTK_LABEL(toplabel), TRUE);
			gtk_label_set_justify(GTK_LABEL(toplabel), GTK_JUSTIFY_CENTER);
			gtk_widget_set_size_request (toplabel, 125, -1);
			gtk_table_attach(GTK_TABLE(table), toplabel, column, column+1, 0, 1, GTK_EXPAND || GTK_SHRINK, GTK_EXPAND || GTK_SHRINK, 1, 1);
		    
			int children_count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store), &iter);
			int t;
		
			for(t=0; t<children_count; t++)
			{	
				//printf("button: %d\n", t);
				GtkTreeIter value_iter, question_iter;
				gchar *value_string, *question_string, *answer_string;
				gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &value_iter, &iter, t);
				gtk_tree_model_get (GTK_TREE_MODEL(store), &value_iter, BYOJ_COLUMN_TOPIC, &value_string, -1);
				gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &question_iter, &value_iter, 0);
				gtk_tree_model_get (GTK_TREE_MODEL(store), &question_iter, BYOJ_COLUMN_QUESTION, &question_string, -1);
				gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &question_iter, &value_iter, 1);
				gtk_tree_model_get (GTK_TREE_MODEL(store), &question_iter, BYOJ_COLUMN_QUESTION, &answer_string, -1);

				GtkWidget *button = gtk_button_new();
				GtkWidget *button_label = gtk_label_new(value_string);
				gtk_container_add(GTK_CONTAINER (button), button_label);
				gtk_label_set_markup(GTK_LABEL(button_label), (gchar *)g_strdup_printf("<span size='30000'>%s</span>", value_string));
			
								
				g_signal_connect ((gpointer) button, "clicked",
						G_CALLBACK (byoj_board_button_clicked),
						NULL);
				
				if (current_block == daily_double_block)
				{
					g_object_set_data(G_OBJECT(button), "value", "Daily Double");	
					g_object_set_data(G_OBJECT(button), "topic", daily_double_topic);	
					g_object_set_data(G_OBJECT(button), "daily_double", GINT_TO_POINTER((gboolean)TRUE));
					g_object_set_data(G_OBJECT(button), "question", daily_double_question);
					g_object_set_data(G_OBJECT(button), "answer", daily_double_answer);
				}
				else
				{
					g_object_set_data(G_OBJECT(button), "topic", topic_string);
					g_object_set_data(G_OBJECT(button), "value", value_string);
					g_object_set_data(G_OBJECT(button), "daily_double", GINT_TO_POINTER((gboolean)FALSE));
					g_object_set_data(G_OBJECT(button), "question", question_string);
					g_object_set_data(G_OBJECT(button), "answer", answer_string);
				}

				GtkWidget * eventbox_button = gtk_event_box_new ();
				gtk_container_add (GTK_CONTAINER (eventbox_button), button);
				gtk_event_box_set_above_child (GTK_EVENT_BOX (eventbox_button), TRUE);
				g_signal_connect ((gpointer) eventbox_button, "button_release_event",
					G_CALLBACK (byoj_eventbox_button_release_event),
					NULL);
				gtk_widget_show_all (eventbox_button);
				
				gtk_table_attach_defaults(GTK_TABLE(table), eventbox_button, column, column+1, t+1, t+2);
			
				current_block +=1;
			}	
			column += 1;
		}
	
		more_topics =  gtk_tree_model_iter_next (GTK_TREE_MODEL(store), &iter);
	}
}

static void
byoj_cell_edited (GtkCellRendererText *cell, gchar *path_string, gchar *new_string, gpointer data)
{
	printf("Cell Edited, %s, %s\n", path_string, new_string);
	          
	GtkTreeModel *tree_model = GTK_TREE_MODEL(data);
	GtkTreeIter iter;
	
	gint column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "column"));
	gtk_tree_model_get_iter_from_string (tree_model, &iter, path_string);

	
	if(gtk_tree_store_iter_is_valid    (GTK_TREE_STORE(tree_model),&iter))
	{
		gchar *old_string;
		gtk_tree_model_get (tree_model, &iter, column, &old_string, -1);
		/*find if string changed to see if we need to store the undo*/
		if(strcmp(old_string, new_string) != 0)
		{
			byoj_undo_add_undo(UNDO_EDIT, &iter);
			gtk_tree_store_set (GTK_TREE_STORE (tree_model), &iter, column, new_string, -1);
		}
	}
	byoj_set_title(TRUE);
}

static void
byoj_fill_template (gboolean Double_BYOJeopardy, GtkWidget *byoj_treeview)
{
	//printf("Build Template: ");
	
	int value_unit = 100;
	if(Double_BYOJeopardy)
	{
		//printf("Double Selected\n");
		value_unit = 200;
	}
	else
	{
		//printf("Single Selected\n");
	}

	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
		
	int t;
	for(t=1; t<=5; t++)
	{
		gchar *topic_string = g_strdup_printf("Category%d", t);
		GtkTreeIter piter = byoj_add_topic(GTK_TREE_STORE(store), topic_string, -1);
		
		int v;
		for(v=1; v<=5; v++)
		{
			int value = v *value_unit;
			gchar *value_string = g_strdup_printf("%d", value);
			gchar *question_string = g_strdup_printf("Question%d", value);
			gchar *answer_string = g_strdup_printf("Answer%d", value);
			
			byoj_add_value(GTK_TREE_STORE(store), piter, value_string, question_string, answer_string, -1);
		}
	}
	//Daily Double	
	byoj_add_daily_double(GTK_TREE_STORE(store), g_strdup("Topic"), g_strdup("Question"), g_strdup("Answer"));
}

static xmlChar *ConvertInput(const char *in, const char *encoding);
static void
byoj_write_file_as_xml(const char *uri, GtkWidget *byoj_treeview)
{
	xmlDocPtr doc = byoj_write_xmldocptr(byoj_treeview);
	xmlSaveFileEnc(uri, doc, MY_ENCODING);
	current_filename = g_strdup_printf("%s", uri);
	xmlFreeDoc(doc);
}

static void
byoj_saveas_file	(GtkWidget *byoj_treeview)
{
		GtkWidget *dialog;
		dialog = create_byoj_filechooserdialog();
		GtkWidget *btnOK = lookup_widget(GTK_WIDGET(dialog), "btnOK");
		gtk_button_set_label (GTK_BUTTON(btnOK), "gtk-save");
		gtk_button_set_use_stock (GTK_BUTTON(btnOK), TRUE);
		gtk_file_chooser_set_action(GTK_FILE_CHOOSER (dialog), GTK_FILE_CHOOSER_ACTION_SAVE);
		if(strcmp(current_filename, "") == 0)
		{
			gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), "Untitled.byj");
		}
		else
		{	
			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), current_filename);		
		}
	
        GtkFileFilter *filefilter_byj = gtk_file_filter_new();
		gtk_file_filter_set_name(filefilter_byj, "BYOJeopardy (*.byj)");
		gtk_file_filter_add_pattern(filefilter_byj, "*.byj");
		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filefilter_byj);
		
        GtkFileFilter *filefilter_all = gtk_file_filter_new();
		gtk_file_filter_set_name(filefilter_all, "All Files (*.*)");
		gtk_file_filter_add_pattern(filefilter_all, "*");
		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filefilter_all);
				
        gtk_window_set_title(GTK_WINDOW(dialog), "Save File");
		gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
		
		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
		{
			gchar *filename;
			filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
			current_filename = g_strdup_printf("%s", filename);
			//byoj_write_file (byoj_treeview);
			//TODO: add chosen file extension if not written already
			
			byoj_write_file_as_xml(filename, byoj_treeview);
			g_free(filename);
		}
		byoj_set_title(FALSE);
	gtk_widget_destroy (dialog);
}

static void
byoj_save_file (GtkWidget *byoj_treeview)
{
	//printf("attempting to save: %s\n", current_filename);
	if(current_filename != NULL && strlen(current_filename)==0)  /*open save as if no file*/
	{
		byoj_saveas_file(byoj_treeview);
	}
	else
	{
		byoj_write_file_as_xml(current_filename, byoj_treeview);
	}
}

static void
byoj_write_file_as_text (GtkWidget *byoj_treeview)
{
	FILE *board;
	board = fopen(current_filename, "w");
	if (board!=NULL)
	{
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));

	GtkTreeIter iter;
	gboolean more_topics = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
	
	while (more_topics) {
		gchar *topic_string;

		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, BYOJ_COLUMN_TOPIC, &topic_string, -1);
		g_strconcat(topic_string, "\n\0");
		fputs (topic_string, board);
		
		int children_count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store), &iter);
		int t;
		
		for(t=0; t<children_count; t++){	
			GtkTreeIter ValueIter;
			  /* made need to change this if questions and answers have a lot of formattin*/
			gchar *value_string, *question_string, *answer_string;
			gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &ValueIter, &iter, t);
			gtk_tree_model_get (GTK_TREE_MODEL(store), &ValueIter, BYOJ_COLUMN_TOPIC, &value_string, 2, &question_string, 4, &answer_string, -1);
			
			gchar *value_output = g_strdup_printf("%s;%s;%s\n", value_string, question_string, answer_string);
			fputs (value_output, board);
		}
		more_topics =  gtk_tree_model_iter_next (GTK_TREE_MODEL(store), &iter);
		}
	fclose (board);
	}
} 

static GtkTreeIter
byoj_get_deletable(GtkWidget *widget)
{
	GtkWidget *byoj_treeview = lookup_widget(widget, "byoj_treeview");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview));
	GtkTreeIter piter;
	if(gtk_tree_selection_get_selected (selection, &store, &piter))
	{
		int iter_type;
		gtk_tree_model_get (store, &piter, BYOJ_COLUMN_TYPE, &iter_type, -1);
		if(iter_type == BYOJ_TOPIC || iter_type == BYOJ_DAILY_DOUBLE)
		{
			printf("deletable topic or daily double\n");
			return piter;
		}
		else if(iter_type == BYOJ_VALUE)
		{
			GtkTreeIter piter2;
			if(gtk_tree_model_iter_parent(store, &piter2, &piter))
			{
			printf("deletable Daily Double1\n");
				int iter_type2;
				gtk_tree_model_get(store, &piter2, BYOJ_COLUMN_TYPE, &iter_type2, -1);
				if(iter_type2 == BYOJ_DAILY_DOUBLE)
				{
					printf("deletable Daily Double\n");
					return piter2;
				}
			}
			printf("deletable value\n");
			return piter;
		}
		else
		{
			//question or answer
			printf("question or answer\n");
		}
	}
}

static gboolean
byoj_iter_deletable(GtkWidget *widget, GtkTreeIter *selected)
{
	GtkWidget *byoj_treeview = lookup_widget(widget, "byoj_treeview");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));

	int iter_type;
	gtk_tree_model_get (store, selected, BYOJ_COLUMN_TYPE, &iter_type, -1);
	if(iter_type == BYOJ_TOPIC || iter_type == BYOJ_DAILY_DOUBLE)
	{
		printf("deletable topic or daily double\n");
		return TRUE;
	}
	else if(iter_type == BYOJ_VALUE)
	{
		GtkTreeIter piter2;
		if(gtk_tree_model_iter_parent(store, &piter2, selected))
		{
			printf("deletable Daily Double1\n");
			int iter_type2;
			gtk_tree_model_get(store, &piter2, BYOJ_COLUMN_TYPE, &iter_type2, -1);
			if(iter_type2 == BYOJ_DAILY_DOUBLE)
			{
				printf("deletable Daily Double\n");
				return TRUE;
			}
		}
		printf("deletable value\n");
		return TRUE;
	}
	else
	{
		//question or answer
		printf("question or answer\n");
		return FALSE;
	}
}

static void
byoj_edit_label (GtkWidget *widget)
{
	GtkWidget *byoj_treeview = lookup_widget(widget, "byoj_treeview");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview));
	GtkTreeIter iter;
	if(gtk_tree_selection_get_selected (selection, &store, &iter))
	{
		int iter_type;
		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, BYOJ_COLUMN_TYPE, &iter_type, -1);
		if(iter_type == BYOJ_QUESTION || iter_type == BYOJ_ANSWER)
		{
			/*Edit question and answer iters*/
			gchar *question_string;
			gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, BYOJ_COLUMN_QUESTION, &question_string, -1);
			//gtk_tree_selection_unselect_all(selection);
			GtkWidget *xhtml_dialog = create_xhtmlite_dialog();
		    //gtk_window_set_transient_for(GTK_WINDOW(xhtml_dialog), GTK_WINDOW(byoj_window_main));
			gchar *xml = g_strdup(question_string);
			xhtmlite_dialog_setup_buffer(xhtml_dialog, xml, TRUE);
			if (gtk_dialog_run (GTK_DIALOG (xhtml_dialog)) == GTK_RESPONSE_OK)
			{
				byoj_undo_add_undo(UNDO_EDIT, &iter);
				GtkTextView *textview = GTK_TEXT_VIEW(lookup_widget(GTK_WIDGET(xhtml_dialog), "xhtml_dialog_textview"));
				gtk_tree_store_set (GTK_TREE_STORE(store), &iter,
						BYOJ_COLUMN_QUESTION, xhtmlite_write_as_markup(textview->buffer),
						BYOJ_COLUMN_QUESTION_EDITABLE, TRUE,
						-1);
			}
			gtk_widget_destroy(xhtml_dialog);
			gtk_tree_selection_select_iter(selection, &iter);
		}
		else
		{
			/*Edit topic and value iters*/
			gtk_tree_view_set_cursor (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store, &iter),
					gtk_tree_view_get_column (GTK_TREE_VIEW(byoj_treeview), BYOJ_COLUMN_TOPIC),  TRUE);
		}
	}
	byoj_set_title(TRUE);
}

static void
byoj_add_topic_clicked (GtkWidget *widget)
{
	GtkWidget *byoj_treeview = lookup_widget(widget, "byoj_treeview");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	GtkTreeIter topic_iter = byoj_add_topic(GTK_TREE_STORE(store), g_strdup("New Topic"), -1);
	
	//select topic
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview));
	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store,&topic_iter), NULL, FALSE, 0.5, 0);
	//gtk_tree_view_expand_row (GTK_TREE_VIEW(view), gtk_tree_model_get_path(model,&iter), TRUE);
	//gtk_tree_selection_select_iter (selection, &topic_iter);
	byoj_undo_add_undo(UNDO_INSERT, &topic_iter);
	gtk_tree_view_set_cursor (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store, &topic_iter),
		gtk_tree_view_get_column (GTK_TREE_VIEW(byoj_treeview), BYOJ_COLUMN_TOPIC),  TRUE);
	
	byoj_set_title(TRUE);
}

static void
byoj_add_value_clicked (GtkWidget *widget)
{
	GtkWidget *byoj_treeview = lookup_widget(widget, "byoj_treeview");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	GtkTreeIter   piter;

	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview));
	if(gtk_tree_selection_get_selected (selection, &store, &piter))
	{
		GtkTreeIter   piter2;
		if(gtk_tree_model_iter_parent(store,  &piter2, &piter))
		{
			piter = piter2;
			if(gtk_tree_model_iter_parent(store,  &piter2, &piter))
			{
				piter = piter2;
			}
		}
		int iter_type;
		gtk_tree_model_get(store, &piter, BYOJ_COLUMN_TYPE, &iter_type, -1);
		if(iter_type == BYOJ_TOPIC)
		{
			GtkTreeIter value_iter = byoj_add_value(GTK_TREE_STORE(store), piter, g_strdup("New Value"), g_strdup("New Question"), g_strdup("New Answer"), -1);
			byoj_undo_add_undo(UNDO_INSERT, &value_iter);
			gtk_tree_view_expand_row (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store,&piter), FALSE);
			gtk_tree_view_expand_row (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store,&value_iter), TRUE);
			gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store,&value_iter), gtk_tree_view_get_column (GTK_TREE_VIEW(byoj_treeview), 0), TRUE, 0.5, 0);
			gtk_tree_view_set_cursor (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store, &value_iter),
							gtk_tree_view_get_column (GTK_TREE_VIEW(byoj_treeview), BYOJ_COLUMN_TOPIC),  TRUE);
		}
	}
	byoj_set_title(TRUE);
}

static void
byoj_add_daily_double_clicked (GtkWidget *widget)
{
	GtkWidget *byoj_treeview = lookup_widget(widget, "byoj_treeview");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview));
	
	GtkTreeIter piter;
	gboolean more_topics = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &piter);
	while (more_topics) 
	{
		int iter_type;
		gtk_tree_model_get(store, &piter, BYOJ_COLUMN_TYPE, &iter_type, -1);
	
		if(iter_type == BYOJ_DAILY_DOUBLE)
		{
			//should not add if there is a label Daily Double
			return ;
		}
		more_topics =  gtk_tree_model_iter_next (GTK_TREE_MODEL(store), &piter);
	}
	GtkTreeIter topic_iter = byoj_add_daily_double(GTK_TREE_STORE(store), g_strdup("New Topic"), g_strdup("New Question"), g_strdup("New Answer"));
	byoj_undo_add_undo(UNDO_INSERT, &topic_iter);
	gtk_tree_view_expand_row (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store, &topic_iter), TRUE);
	GtkTreeIter value_iter;
	gtk_tree_model_iter_nth_child(store, &value_iter, &topic_iter, 0);
	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store, &value_iter),  gtk_tree_view_get_column (GTK_TREE_VIEW(byoj_treeview), BYOJ_COLUMN_TOPIC), TRUE, 0.5, 0);
	gtk_tree_view_set_cursor (GTK_TREE_VIEW(byoj_treeview), gtk_tree_model_get_path(store, &value_iter),
				  gtk_tree_view_get_column (GTK_TREE_VIEW(byoj_treeview), BYOJ_COLUMN_TOPIC),  TRUE);

	byoj_set_title(TRUE);
}

static void
byoj_delete (GtkWidget *widget)
{
	GtkTreeIter iter_to_delete = byoj_get_deletable(widget);
	GtkWidget *byoj_treeview = lookup_widget(widget, "byoj_treeview");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	if(&iter_to_delete != NULL)
	{
			byoj_undo_add_undo(UNDO_DELETE, &iter_to_delete);
			gtk_tree_store_remove(GTK_TREE_STORE(store), &iter_to_delete);
	}
}

static void
byoj_set_edit_mode_on(GtkWidget *widget)
{
	GtkWidget *byoj_image = lookup_widget(widget, "byoj_image");
	GtkWidget *byoj_table = lookup_widget(widget, "byoj_table");
	GtkWidget *byoj_hbox_treeview = lookup_widget(widget, "byoj_hbox_treeview");
	GtkWidget *byoj_menuitem_save = lookup_widget(widget, "byoj_menuitem_save");
	GtkWidget *byoj_menuitem_saveas = lookup_widget(widget, "byoj_menuitem_saveas");
	GtkWidget *byoj_menuitem_export = lookup_widget(widget, "byoj_menuitem_export");
	GtkWidget *byoj_menu_edit = lookup_widget(widget, "byoj_menu_edit");
	GtkWidget *byoj_menuitem_preferences = lookup_widget(widget, "byoj_menuitem_preferences");
	GtkWidget *byoj_seperator5 = lookup_widget(widget, "byoj_separator5");
	GtkWidget *byoj_menuitem_rebuild_board = lookup_widget(widget, "byoj_menuitem_rebuild_board");

	gtk_widget_set_sensitive (byoj_menuitem_save, TRUE);
	gtk_widget_set_sensitive(byoj_menuitem_saveas, TRUE);
	gtk_widget_set_sensitive(byoj_menuitem_export, TRUE);
	gtk_widget_show_all (byoj_menu_edit);
	gtk_widget_show (byoj_hbox_treeview);
	gtk_widget_hide (byoj_table);
	gtk_widget_hide (byoj_image);
	gtk_widget_hide (byoj_menuitem_preferences);
	gtk_widget_hide (byoj_seperator5);
	gtk_widget_set_sensitive(byoj_menuitem_rebuild_board, TRUE);
}

/**
* ConvertInput:
* @in: string in a given encoding
* @encoding: the encoding used
*
* Converts @in into UTF-8 for processing with libxml2 APIs
*
* Returns the converted UTF-8 string, or NULL in case of error.
*/
static xmlChar *
byoj_convert_input(const char *in, const char *encoding)
{
	xmlChar *out;
	int ret;
	int size;
	int out_size;
	int temp;
	xmlCharEncodingHandlerPtr handler;

	if (in == 0)
		return 0;

	handler = xmlFindCharEncodingHandler(encoding);

	if (!handler) {
		printf("ConvertInput: no encoding handler found for '%s'\n",
		encoding ? encoding : "");
		return 0;
	}

	size = (int) strlen(in) + 1;
	out_size = size * 2 - 1;
	out = (unsigned char *) xmlMalloc((size_t) out_size);

	if (out != 0) {
		temp = size - 1;
		ret = handler->input(out, &out_size, (const xmlChar *) in, &temp);
		if ((ret < 0) || (temp - size + 1)) {
			if (ret < 0) {
				//printf("ConvertInput: conversion wasn't successful.\n");
			} else {
				//printf
						("ConvertInput: conversion wasn't successful. converted: %i octets.\n",
						temp);
			}

			xmlFree(out);
			out = 0;
		} else {
			out = (unsigned char *) xmlRealloc(out, out_size + 1);
			out[out_size] = 0;  /*null terminating out */
		}
	} else {
		//printf("ConvertInput: no mem\n");
	}
	return out;
}

xmlDocPtr
byoj_write_xmldocptr(GtkWidget *byoj_treeview)
{
	int rc;
	xmlTextWriterPtr writer;
	xmlChar *tmp;
	xmlDocPtr doc;


	/* Create a new XmlWriter for DOM, with no compression. */
	writer = xmlNewTextWriterDoc(&doc, 0);
	if (writer == NULL) {
		printf("testXmlwriterDoc: Error creating the xml writer\n");
		return;
	}
	
/* Start the document with the xml default for the version,
	* encoding ISO 8859-1 and the default for the standalone
* declaration. */
	rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
	if (rc < 0) {
		//printf("testXmlwriterFilename: Error at xmlStartDocument\n");
		return;
	}

/* Start an element named "EXAMPLE". Since thist is the first
* element, this will be the root element of the document. */
	rc = xmlTextWriterStartElement(writer, BAD_CAST "byojeopardy");
	if (rc < 0) {
		//printf("testXmlwriterFilename: Error at xmlStartElement\n");
		return;
	}
	/* Add an attribute with name "version" and value "1.0" to ORDER. */
	rc = xmlTextWriterWriteAttribute(writer, BAD_CAST "version",
					BAD_CAST "1.1");
	if (rc < 0) {
		//printf("testXmlwriterFilename: Error at xmlTextWriterWriteAttribute\n");
		return;
	}

	/* Write a comment as child of EXAMPLE.
		* Please observe, that the input to the xmlTextWriter functions
		* HAS to be in UTF-8, even if the output XML is encoded
	* in iso-8859-1 */
	// 	tmp = ConvertInput("This is a comment with special chars: <�>",
	// 			MY_ENCODING);
	// 	rc = xmlTextWriterWriteComment(writer, tmp);
	// 	if (rc < 0) {
	// 		//printf
	// 				("testXmlwriterFilename: Error at xmlTextWriterWriteComment\n");
	// 		return;
	// 	}
	// 	if (tmp != NULL) xmlFree(tmp);

	
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));

	GtkTreeIter iter;
	gboolean more_topics = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
	
	while (more_topics) {
		/* Start an element named "topic" as child of byojeopardy. */
		rc = xmlTextWriterStartElement(writer, BAD_CAST "topic");
		if (rc < 0) {
			//printf("testXmlwriterFilename: Error at xmlTextWriterStartElement\n");
			return;
		}
				
		gchar * topic_string;

		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 0, &topic_string, -1);
				
		/* Write an element named "topic_name" as child of topic. */
		tmp = byoj_convert_input(topic_string, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "topic_name", tmp);
		if (rc < 0) {
			//printf
					("testXmlwriterFilename: Error at xmlTextWriterWriteFormatElement\n");
			return;
		}
		
		int children_count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store), &iter);
		int t;
		
		for(t=0; t<children_count; t++){	
			/* Start an element named "value" as child of byojeopardy. */
			rc = xmlTextWriterStartElement(writer, BAD_CAST "value");
			if (rc < 0) {
				//printf
						("testXmlwriterFilename: Error at xmlTextWriterStartElement\n");
				return;
			}
			
			GtkTreeIter value_iter, question_iter;
			gchar * value_string, * question_string, * answer_string;
			gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &value_iter, &iter, t);
			gtk_tree_model_get (GTK_TREE_MODEL(store), &value_iter, BYOJ_COLUMN_TOPIC, &value_string, -1);
			gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &question_iter, &value_iter, 0);
			gtk_tree_model_get (GTK_TREE_MODEL(store), &question_iter, BYOJ_COLUMN_TYPE, &question_string, -1);
			gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &question_iter, &value_iter, 1);
			gtk_tree_model_get (GTK_TREE_MODEL(store), &question_iter, BYOJ_COLUMN_TYPE, &answer_string, -1);
	
	
			/* Write an element named "amount" as child of value. */
			tmp = byoj_convert_input(value_string, MY_ENCODING);
			rc = xmlTextWriterWriteElement(writer, BAD_CAST "amount", tmp);
			if (rc < 0) {
				//printf("testXmlwriterFilename: Error at xmlTextWriterWriteFormatElement\n");
				return;
			}
			/* Write an element named "question" as child of value. */
			tmp = byoj_convert_input(question_string, MY_ENCODING);
			rc = xmlTextWriterWriteElement(writer, BAD_CAST "question", tmp);
			if (rc < 0) {
				//printf("testXmlwriterFilename: Error at xmlTextWriterWriteFormatElement\n");
				return;
			}
			/* Write an element named "answer" as child of value. */
			tmp = byoj_convert_input(answer_string, MY_ENCODING);
			//xmlURIEscape(tmp);
			//printf("Answer:%s", tmp);
			rc = xmlTextWriterWriteElement(writer, BAD_CAST "answer", tmp);
			if (rc < 0) {
				//printf("testXmlwriterFilename: Error at xmlTextWriterWriteFormatElement\n");
				return;
			}
			
			/* Close the element named value. */
			rc = xmlTextWriterEndElement(writer);
			if (rc < 0) {
				//printf("testXmlwriterFilename: Error at xmlTextWriterEndElement\n");
				return;
			}
		}
		
		/* Close the element named topic. */
		rc = xmlTextWriterEndElement(writer);
		if (rc < 0) {
			//printf("testXmlwriterFilename: Error at xmlTextWriterEndElement\n");
			return;
		}
		
		more_topics =  gtk_tree_model_iter_next (GTK_TREE_MODEL(store), &iter);
	}
	

/* Close all the remaining elements */
	rc = xmlTextWriterEndDocument(writer);
	if (rc < 0) {
		printf("XmlwriterFilename: Error at xmlTextWriterEndDocument\n");
		return;
	}

	xmlFreeTextWriter(writer);
	return doc;
}

gboolean
byoj_read_file(gchar *filename, GtkWidget *widget)
{
        if(!byoj_read_xml_file(filename, widget))
		{
			printf("Error: Failed to open the file as XML format. Attempting to open file as old text format.");
			if(!byoj_open_text_file(filename, widget))
			{
				printf("Error: Failed to open the file as old text format.");
				return FALSE;
			}
		}
		printf("File opened successfully!\n"); 
		
        GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(widget), "byoj_treeview");
		byoj_build_board(byoj_treeview);

		GtkWidget *byoj_menuitem_edit = lookup_widget(GTK_WIDGET(widget), "byoj_menuitem_edit");
		GtkWidget *byoj_menuitem_rebuild_board = lookup_widget(GTK_WIDGET(widget), "byoj_menuitem_rebuild_board");
		gtk_widget_set_sensitive (byoj_menuitem_edit, TRUE);
		gtk_widget_set_sensitive(byoj_menuitem_rebuild_board, TRUE);
		byoj_show_board(widget);
		
		current_filename = g_strdup_printf("%s", filename); 

  return TRUE;
} 

GtkTreeIter
byoj_add_topic (GtkTreeStore *store, gchar *topic_string, gint position)
{
	printf("Adding Topic: %s\n", topic_string);
	GtkTreeIter topic_iter;
	gtk_tree_store_insert(store,&topic_iter,NULL,position);
	gtk_tree_store_set (store, &topic_iter,
				BYOJ_COLUMN_TOPIC, topic_string,
			    BYOJ_COLUMN_TOPIC_EDITABLE, TRUE,
				BYOJ_COLUMN_TYPE, BYOJ_TOPIC,
				-1);
	return topic_iter;
}

GtkTreeIter 
byoj_add_value (GtkTreeStore *store, GtkTreeIter topic_iter, gchar *value_string, gchar *question_string, gchar *answer_string, gint position)
{
	printf("Adding Value: %s\n", value_string);
	GtkTreeIter value_iter;
	gtk_tree_store_insert(store, &value_iter, &topic_iter, position);
	gtk_tree_store_set (store, &value_iter,
			    BYOJ_COLUMN_TOPIC, value_string,
			    BYOJ_COLUMN_TOPIC_EDITABLE, TRUE,
				BYOJ_COLUMN_TYPE, BYOJ_VALUE,
			    -1);
	
	GtkTreeIter question_iter;
	gtk_tree_store_append(store, &question_iter, &value_iter);
	gtk_tree_store_set (store, &question_iter,
			    BYOJ_COLUMN_QUESTION, question_string,
			    BYOJ_COLUMN_QUESTION_EDITABLE, TRUE,
				BYOJ_COLUMN_TYPE, BYOJ_QUESTION,
			    -1);

	gtk_tree_store_append(store, &question_iter, &value_iter);
	gtk_tree_store_set (store, &question_iter,
			    BYOJ_COLUMN_QUESTION, answer_string,
			    BYOJ_COLUMN_QUESTION_EDITABLE, TRUE,
				BYOJ_COLUMN_TYPE, BYOJ_ANSWER,
			    -1);
	return value_iter;
}

GtkTreeIter 
byoj_add_daily_double_topic (GtkTreeStore *store)
{
	printf("Adding Daily Double\n");
	GtkTreeIter topic_iter;
	gtk_tree_store_append (store, &topic_iter, NULL);  /* Acquire an iterator */
	gtk_tree_store_set (store, &topic_iter,
			    BYOJ_COLUMN_TOPIC, "Daily Double",
			    BYOJ_COLUMN_TOPIC_EDITABLE, FALSE,
				BYOJ_COLUMN_TYPE, BYOJ_DAILY_DOUBLE,
			    -1);

	return topic_iter;
}

GtkTreeIter 
byoj_add_daily_double (GtkTreeStore *store, gchar *value_string, gchar *question_string, gchar *answer_string)
{
	printf("Adding Daily Double\n");
	GtkTreeIter topic_iter = byoj_add_daily_double_topic(store);
	GtkTreeIter value_iter = byoj_add_value(store, topic_iter, value_string, question_string, answer_string, -1);
	return topic_iter;
}

void
byoj_menuitem_open_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkWidget *dialog = create_byoj_filechooserdialog();
	GtkWidget *btnOK = lookup_widget(GTK_WIDGET(dialog), "btnOK");
	gtk_button_set_label (GTK_BUTTON(btnOK), "gtk-open");
	gtk_button_set_use_stock (GTK_BUTTON(btnOK), TRUE);
	gtk_file_chooser_set_action(GTK_FILE_CHOOSER (dialog), GTK_FILE_CHOOSER_ACTION_OPEN);

    GtkFileFilter *filefilter_byj = gtk_file_filter_new();
	gtk_file_filter_set_name(filefilter_byj, "BYOJeopardy (*.byj)");
	gtk_file_filter_add_pattern(filefilter_byj, "*.byj");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filefilter_byj);
		
    GtkFileFilter *filefilter_all = gtk_file_filter_new();
	gtk_file_filter_set_name(filefilter_all, "All Files (*.*)");
	gtk_file_filter_add_pattern(filefilter_all, "*");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filefilter_all);

	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), current_filename);

	gtk_window_set_title(GTK_WINDOW(dialog), "Open File");

	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
	{
		gtk_widget_hide(dialog);
		current_filename = g_strdup_printf("");
		gchar *filename;
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		
        byoj_read_file(filename, byoj_window_main);
	    byoj_set_title(FALSE);
		printf("completed opening: %s\n", filename);	
		g_free (filename);
		byoj_undo_clear_undos();
	}
	gtk_widget_destroy (dialog);
}

void
byoj_menuitem_edit_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_set_edit_mode_on(GTK_WIDGET(menuitem));
}


void
byoj_menuitem_exit_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	gtk_main_quit ();
}


void
byoj_menuitem_fullscreen_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	if( gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(menuitem)))
{
	//printf("request fullscreen\n");
	gtk_window_fullscreen(GTK_WINDOW(byoj_window_main));
}
else
{
	//printf("request unfullscreen\n");
	gtk_window_unfullscreen(GTK_WINDOW(byoj_window_main));
}
}

void
byoj_menuitem_about_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{		
	GtkWidget *dialog;
	dialog = create_byoj_dialog_about();
	gtk_window_set_icon(GTK_WINDOW (dialog), gtk_window_get_icon(GTK_WINDOW(byoj_window_main)));
	gtk_dialog_run (GTK_DIALOG (dialog));
}

gboolean
byoj_window_main_on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
gtk_main_quit ();
return FALSE;
}

void
byoj_menuitem_new_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
current_filename = g_strdup_printf("");
GtkWidget *dialog;
dialog = create_byoj_dialog_template ();
gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(byoj_window_main));


if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
{
	gtk_widget_hide(dialog);
	/*GtkWidget *FormEdit;
	FormEdit = create_FormEdit ();*/

	//Setup list
	GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(menuitem), "byoj_treeview");
	
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	gtk_tree_store_clear(GTK_TREE_STORE(store));
	
	GtkWidget *byoj_radiobutton_double = lookup_widget(GTK_WIDGET(dialog), "byoj_radiobutton_double");
	GtkWidget *byoj_radiobutton_none = lookup_widget(GTK_WIDGET(dialog), "byoj_radiobutton_none");
	
	gboolean Double = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(byoj_radiobutton_double));
	gboolean NoTemplate = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(byoj_radiobutton_none));
	if(!NoTemplate)
	{
		byoj_fill_template(Double, byoj_treeview);
	}
	byoj_build_board(byoj_treeview);
	
	byoj_set_title(TRUE);
	byoj_set_edit_mode_on(GTK_WIDGET(menuitem));
	GtkWidget *byoj_menuitem_edit = lookup_widget(GTK_WIDGET(menuitem), "byoj_menuitem_edit");
	gtk_widget_set_sensitive (byoj_menuitem_edit, TRUE);

	byoj_undo_clear_undos();
}
	gtk_widget_destroy(dialog);
}


static gboolean   
byoj_treeview_button_press_event (GtkWidget *widget, GdkEventButton *event, GtkWidget *byoj_menu_edit_menu)
{
	if (event->button == 3)
	{
		gtk_menu_popup (GTK_MENU(byoj_menu_edit_menu), NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time());
	}
	else
	{
		return FALSE;
	}
}

void
byoj_setup_treeview (GtkWidget *byoj_treeview)
{
	//delete old byoj_treeview store and columns
	//GtkTreeModel *oldstore = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	//gtk_tree_store_clear();
	//gtk_widget_destroy(GTK_WIDGET(oldstore));
	//gtk_tree_view_remove_column(GTK_TREE_VIEW(byoj_treeview), gtk_tree_view_get_column(GTK_TREE_VIEW(byoj_treeview),1));
	//gtk_tree_view_remove_column(GTK_TREE_VIEW(byoj_treeview), gtk_tree_view_get_column(GTK_TREE_VIEW(byoj_treeview),0));
	
	//right-click menu
	GtkWidget *byoj_menu_edit_menu = lookup_widget(byoj_treeview, "byoj_menu_edit_menu");
	g_signal_connect (byoj_treeview, "button-press-event", G_CALLBACK (byoj_treeview_button_press_event), byoj_menu_edit_menu);
	g_signal_connect (byoj_treeview, "button-release-event", G_CALLBACK (byoj_treeview_button_press_event), byoj_menu_edit_menu);

	//create new byoj_treeview store and columns
	GtkTreeStore *store = gtk_tree_store_new (5,
				G_TYPE_STRING,
				G_TYPE_BOOLEAN,
				G_TYPE_STRING,
				G_TYPE_BOOLEAN,
				G_TYPE_INT, -1);
	gtk_tree_view_set_model(GTK_TREE_VIEW(byoj_treeview), GTK_TREE_MODEL(store));
	
	GtkCellRenderer *TopicRender = gtk_cell_renderer_text_new ();
	g_object_set_data (G_OBJECT (TopicRender), "column", GINT_TO_POINTER(BYOJ_COLUMN_TOPIC));
	g_signal_connect (TopicRender, "edited", G_CALLBACK (byoj_cell_edited), store);
	
	GtkTreeViewColumn *TopicCol = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title(TopicCol, "Topic");
	gtk_tree_view_column_pack_start (TopicCol, TopicRender, TRUE);
	gtk_tree_view_column_add_attribute (TopicCol, TopicRender, "text", BYOJ_COLUMN_TOPIC);
	gtk_tree_view_column_add_attribute (TopicCol, TopicRender, "editable", BYOJ_COLUMN_TOPIC_EDITABLE);
	gtk_tree_view_column_set_expand (TopicCol, FALSE);
	gtk_tree_view_column_set_resizable(TopicCol, TRUE);
	gtk_tree_view_column_set_sort_column_id(TopicCol, 0);
	gtk_tree_view_append_column (GTK_TREE_VIEW(byoj_treeview), TopicCol);
	//gtk_tree_view_column_set_reorderable(TopicCol, TRUE);
	//gtk_tree_view_column_set_visible (TopicCol, TRUE);

	GtkCellRenderer *QuestionRender = gtk_cell_renderer_text_new ();
	g_object_set_data (G_OBJECT (QuestionRender), "column", GINT_TO_POINTER(BYOJ_COLUMN_QUESTION));
	g_signal_connect(QuestionRender, "edited", (GCallback)byoj_cell_edited, store);
	GtkTreeViewColumn *QuestionColumn = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title(QuestionColumn, "Question / Answer");
	gtk_tree_view_column_pack_start (QuestionColumn, QuestionRender, TRUE);
	gtk_tree_view_column_add_attribute (QuestionColumn, QuestionRender, "text", BYOJ_COLUMN_QUESTION);
	gtk_tree_view_column_add_attribute (QuestionColumn, QuestionRender, "editable", BYOJ_COLUMN_QUESTION_EDITABLE);
	gtk_tree_view_column_set_expand (QuestionColumn, TRUE);
	gtk_tree_view_column_set_resizable(QuestionColumn, TRUE);
	gtk_tree_view_column_set_sort_column_id(QuestionColumn, BYOJ_COLUMN_QUESTION);
	gtk_tree_view_append_column (GTK_TREE_VIEW(byoj_treeview), QuestionColumn);

	gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(byoj_treeview), TRUE);
	//gtk_widget_show_all(byoj_treeview);
}

void
byoj_menuitem_show_board_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_show_board(GTK_WIDGET(menuitem));
}

void
byoj_button_home_clicked (GtkButton *button, gpointer user_data)
{
	gtk_widget_hide (byoj_window_question);
}

void
byoj_button_question_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *byoj_button_label_question = lookup_widget(GTK_WIDGET(button), "byoj_button_label_question");
	GtkWidget *byoj_textview_question = lookup_widget(GTK_WIDGET(button), "byoj_textview_question");

	gchar *question = g_object_get_data(G_OBJECT(byoj_window_question), "question");
	gchar *answer = g_object_get_data(G_OBJECT(byoj_window_question), "answer");

	const gchar *btnlabel = gtk_label_get_label(GTK_LABEL(byoj_button_label_question));
	gchar *buf, *markup;

	if(strstr(btnlabel, "Answer")!=NULL || strstr(btnlabel, "Start Daily Double")!=NULL && byoj_answer_in_form_of_question)
	{
		markup = g_strdup_printf("<span size='20000' color='#333'>Question</span>");
		gchar *relative_size = byoj_question_size(answer);
    	buf = g_strdup_printf("<center><byojeopardy_%s>%s</byojeopardy_%s></center>", relative_size, answer, relative_size);
	}
	else
	{
		markup = g_strdup_printf("<span size='20000' color='#333'>Answer</span>");
		gchar *relative_size = byoj_question_size(question);
    	buf = g_strdup_printf("<center><byojeopardy_%s>%s</byojeopardy_%s></center>", relative_size, question, relative_size);
	}
	gtk_label_set_markup(GTK_LABEL(byoj_button_label_question), markup);
	xhtmlite_import(GTK_TEXT_VIEW(byoj_textview_question)->buffer, buf, TRUE);
}

void
byoj_menuitem_save_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(menuitem), "byoj_treeview");
	byoj_save_file(byoj_treeview);
}


void
byoj_menuitem_saveas_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(menuitem), "byoj_treeview");
	byoj_saveas_file(byoj_treeview);
}


void
byoj_toolbutton_save_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(toolbutton), "byoj_treeview");
	byoj_save_file(byoj_treeview);
}


void
byoj_toolbutton_saveas_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(toolbutton), "byoj_treeview");
	byoj_saveas_file(byoj_treeview);
}

void
byoj_toolbutton_edit_label_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	byoj_edit_label (GTK_WIDGET(toolbutton));
}


void
byoj_toolbutton_add_topic_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	byoj_add_topic_clicked (GTK_WIDGET(toolbutton));
}


void
byoj_toolbutton_add_value_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	byoj_add_value_clicked (GTK_WIDGET(toolbutton));
}


void
byoj_toolbutton_add_daily_double_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	byoj_add_daily_double_clicked (GTK_WIDGET(toolbutton));
}

void
byoj_toolbutton_delete_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	byoj_delete (GTK_WIDGET(toolbutton));
}


void
byoj_menuitem_rebuild_board_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_build_board(byoj_window_main);
	byoj_show_board(GTK_WIDGET(menuitem));
}

void
byoj_menuitem_export_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_export_dialog_open(GTK_WIDGET(menuitem));
}

void
byoj_menuitem_preferences_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{

}

void
byoj_menuitem_undo_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_undo_undo ();
}

void
byoj_menuitem_redo_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_undo_redo ();
}

void
byoj_menuitem_edit_label_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_edit_label (GTK_WIDGET(menuitem));
}

void
byoj_menuitem_add_topic_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_add_topic_clicked (GTK_WIDGET(menuitem));
}


void
byoj_menuitem_add_value_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_add_value_clicked (GTK_WIDGET(menuitem));
}


void
byoj_menuitem_add_daily_double_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_add_daily_double_clicked (GTK_WIDGET(menuitem));
}


void
byoj_menuitem_delete_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	byoj_delete (GTK_WIDGET(menuitem));
}

static undo *byoj_cutpaste = NULL;

void
byoj_menuitem_cut_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkTreeIter iter;
	GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(menuitem), "byoj_treeview");
	GtkTreeModel *tree_model = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview));

	if(gtk_tree_selection_get_selected (selection, &tree_model, &iter))
	{
		if(byoj_iter_deletable(GTK_WIDGET(menuitem), &iter))
		{
			byoj_cutpaste = malloc(sizeof(undo));
			byoj_undo_store_data_for_delete(tree_model, &iter, byoj_cutpaste);
			byoj_undo_add_undo(UNDO_DELETE, &iter);
			gtk_tree_store_remove(GTK_TREE_STORE(tree_model), &iter);
		}
		else //BYOJ_QUESTION or BYOJ_ANSWER
		{
			byoj_cutpaste = malloc(sizeof(undo));
			byoj_undo_store_data_for_delete(tree_model, &iter, byoj_cutpaste);
			byoj_undo_add_undo(UNDO_EDIT, &iter);
			int iter_type;
			gtk_tree_model_get(tree_model, &iter, BYOJ_COLUMN_TYPE, &iter_type, -1);
			if(iter_type == BYOJ_QUESTION)
			{
				gtk_tree_store_set(GTK_TREE_STORE(tree_model), &iter, BYOJ_COLUMN_QUESTION, g_strdup("Question"), -1);
			}
			else
			{
				gtk_tree_store_set(GTK_TREE_STORE(tree_model), &iter, BYOJ_COLUMN_QUESTION, g_strdup("Answer"), -1);
			}
		}
		GtkWidget *byoj_menuitem_paste = lookup_widget(GTK_WIDGET(menuitem), "byoj_menuitem_paste");
		gtk_widget_set_sensitive (byoj_menuitem_paste, TRUE);
	}
}


void
byoj_menuitem_copy_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkTreeIter iter;
	GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(menuitem), "byoj_treeview");
	GtkTreeModel *tree_model = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview));

	if(gtk_tree_selection_get_selected (selection, &tree_model, &iter))
	{
		byoj_iter_deletable(GTK_WIDGET(menuitem), &iter);
		byoj_cutpaste = malloc(sizeof(undo));
		byoj_undo_store_data_for_delete(tree_model, &iter, byoj_cutpaste);
		GtkWidget *byoj_menuitem_paste = lookup_widget(GTK_WIDGET(menuitem), "byoj_menuitem_paste");
		gtk_widget_set_sensitive (byoj_menuitem_paste, TRUE);
	}
}

int
byoj_has_daily_double(GtkTreeView *byoj_treeview)
{
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));

	GtkTreeIter iter;
	gboolean more_topics = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
	while (more_topics) {
 		int iter_type;
		gtk_tree_model_get (store, &iter, BYOJ_COLUMN_TYPE, &iter_type, -1);
		if(iter_type == BYOJ_DAILY_DOUBLE)
		{
			return 1;
		}
		more_topics =  gtk_tree_model_iter_next (GTK_TREE_MODEL(store), &iter);
	}
return 0;
}

static void
byoj_paste_question_or_answer(GtkWidget *widget)
{
	GtkWidget *byoj_treeview = lookup_widget(widget, "byoj_treeview");
	GtkTreeModel *tree_model = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));

	//Selection
	GtkTreeIter iter_selected;
	int iter_selected_type;
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview));
	if(gtk_tree_selection_get_selected(selection, &tree_model, &iter_selected))
	{
		printf("BYOJ_QUESTION or BYOJ_ANSWER\n");
		gtk_tree_model_get (tree_model, &iter_selected, BYOJ_COLUMN_TYPE, &iter_selected_type, -1);
		if(iter_selected_type == BYOJ_QUESTION || iter_selected_type == BYOJ_ANSWER)
		{
			printf("iter_selected_type:BYOJ_QUESTION or BYOJ_ANSWER\n");
			byoj_undo_add_undo(UNDO_EDIT, &iter_selected);
			gtk_tree_store_set(GTK_TREE_STORE(tree_model), &iter_selected, BYOJ_COLUMN_QUESTION, g_strdup(byoj_cutpaste->column_string), -1);
		}
		else if(iter_selected_type == BYOJ_VALUE)
		{
			int child;
			if(byoj_cutpaste->iter_type == BYOJ_QUESTION)
			{
				child = 0;
			}
			else
			{
				child = 1;
			}
			GtkTreeIter iter_child;
			if(gtk_tree_model_iter_nth_child (tree_model, &iter_child, &iter_selected, child))
			{
				byoj_undo_add_undo(UNDO_EDIT, &iter_child);
				gtk_tree_store_set(GTK_TREE_STORE(tree_model), &iter_child, BYOJ_COLUMN_QUESTION, g_strdup(byoj_cutpaste->column_string), -1);		
			}
		}
	}
	
}


void
byoj_menuitem_paste_on_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	if(byoj_cutpaste != NULL)
	{
		GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(menuitem), "byoj_treeview");
		GtkTreeModel *tree_model = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));
		
		if(byoj_cutpaste->iter_type == BYOJ_QUESTION || byoj_cutpaste->iter_type == BYOJ_ANSWER)
		{
			byoj_paste_question_or_answer(GTK_WIDGET(menuitem));
			return;
		}
	
		//Selection
		GtkTreeIter iter_selected;
		GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview));
		GtkTreePath *path_selected;
		int topic_position = 0;
		int value_position = -1;
		int iter_selected_type;
		if(gtk_tree_selection_get_selected(selection, &tree_model, &iter_selected))
		{
			path_selected = gtk_tree_model_get_path (tree_model, &iter_selected);
			gtk_tree_model_get (tree_model, &iter_selected, BYOJ_COLUMN_TYPE, &iter_selected_type, -1);
			topic_position = byoj_get_topic_position(tree_model, &iter_selected);
			if(iter_selected_type == BYOJ_VALUE || iter_selected_type == BYOJ_QUESTION || iter_selected_type == BYOJ_ANSWER)
			{
				value_position = byoj_get_value_position(tree_model, &iter_selected);
			}

			//Do not allow objects to be pasted in a daily double
			GtkTreeIter iter_parent;
			if(gtk_tree_model_iter_parent(tree_model, &iter_parent, &iter_selected))
			{
				gtk_tree_model_get (tree_model, &iter_parent, BYOJ_COLUMN_TYPE, &iter_selected_type, -1);
			}
			if(iter_selected_type == BYOJ_DAILY_DOUBLE)
			{
				return;
			}
			else if(iter_selected_type == BYOJ_TOPIC)
			{
				gtk_tree_view_expand_row (GTK_TREE_VIEW(byoj_treeview), path_selected, FALSE);
			}
		}
	
		GtkTreeIter topic_iter;
		gboolean has_topic_iter = FALSE;
		if(byoj_cutpaste->iter_type == BYOJ_TOPIC)
		{
			has_topic_iter = TRUE;
			topic_iter = byoj_add_topic(GTK_TREE_STORE(tree_model), byoj_cutpaste->topic_data->topic_string, byoj_cutpaste->topic_data->position);
		}
		else if(byoj_cutpaste->iter_type == BYOJ_DAILY_DOUBLE)
		{
			//check to see if there is a daily double first
			if(byoj_has_daily_double(GTK_TREE_VIEW(byoj_treeview)))
			{
				return;
			}
			has_topic_iter = TRUE;
			topic_iter = byoj_add_daily_double_topic(GTK_TREE_STORE(tree_model));
		}
		else if(byoj_cutpaste->iter_type == BYOJ_DAILY_DOUBLE)
		{
			printf("byoj_cutpaste type == BYOJ_DAILY_DOUBLE\n");
			return;
		}
		else if(byoj_cutpaste->iter_type == BYOJ_VALUE)
		{
			printf("iter path:%d\n",byoj_cutpaste->topic_data->position); 
			gtk_tree_model_get_iter_from_string(tree_model, &topic_iter, g_strdup_printf("%d", topic_position));
		}
	
		int n = 0;
		value_struct *value_data = g_list_nth_data (byoj_cutpaste->topic_data->values, n);
		GtkTreeIter value_iter;
		while (value_data != NULL)
		{
			value_iter = byoj_add_value(GTK_TREE_STORE(tree_model), topic_iter, value_data->value_string, value_data->question_string, value_data->answer_string, value_position);
			n+=1;
			value_data = g_list_nth_data (byoj_cutpaste->topic_data->values, n);
		}

		//GET Selection
		if(&topic_iter != NULL)
		{
			path_selected = gtk_tree_model_get_path (tree_model, &topic_iter);
			gtk_tree_view_expand_row (GTK_TREE_VIEW(byoj_treeview), path_selected, FALSE);
		}
		if(&value_iter != NULL)
		{
			path_selected = gtk_tree_model_get_path (tree_model, &value_iter);
			gtk_tree_view_expand_row (GTK_TREE_VIEW(byoj_treeview), path_selected, FALSE);
		}

		gtk_tree_selection_select_path (selection, path_selected);
	
		if(has_topic_iter)
		{
			byoj_undo_add_undo(UNDO_INSERT, &topic_iter);
		}
		else
		{
			byoj_undo_add_undo(UNDO_INSERT, &value_iter);
		}
	}
}

int
byoj_count_topics()
{

	//GtkWidget *byoj_treeview = lookup_widget(GTK_WIDGET(widget), "byoj_treeview");
	//GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview));

	// 
	//GtkTreeIter iter;
	//gboolean more_topics = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
//TODO: Complete this!!!!
}

int
byoj_count_maximum_values()
{
	
}
